/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;

/**
 * BufferedPane double-buffers graphics.
 */
public class BufferedPane
  extends java.awt.Panel
{
////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
  
  /**
   * Override update() to call paint() to reduce flicker.
   */
  public final void update(Graphics g) 
  { 
    paint(g);
  }  
  
  /**
   * Double-buffer graphics to reduce flicker.
   */
  public final void paint(Graphics graphics)
  {
    int w = getSize().width;
    int h = getSize().height;    
        
    // Create new buffer if neccesary
    if (buffer == null || buffer.getWidth(null) != w || buffer.getHeight(null) != h)
      buffer = createImage(w, h);
      
    Graphics2D g = (Graphics2D)buffer.getGraphics();    
    g.setRenderingHint(
      RenderingHints.KEY_TEXT_ANTIALIASING, 
      RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    
    doPaint(g);
    graphics.drawImage(buffer, 0, 0, null);
    g.dispose();
  }
  
  /**
   * Put paint code here. 
   */
  public void doPaint(Graphics2D g)
  {    
  }  
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private Image buffer;
}



