/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;
import java.awt.image.*;

/**
 * DetailsPane displays details of the currently selected game.
 */
public class DetailsPane
  extends Widget
  implements SelectionListener
{
////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////
  
  /**
   * Create new ListPane using the specified RomManager.
   */  
  public DetailsPane()
  {    
  }
  
////////////////////////////////////////////////////////////////
// Layout
////////////////////////////////////////////////////////////////

  /**
   * Return the preferred size for this widget.
   */
  public Dimension getPreferredSize()
  {
    return dim;
  }
  private static final Dimension dim = new Dimension(450,450);
  
////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
  
  /**
   * Paint our list.
   */
  public void paint(Graphics2D g)
  {
    int w = getBounds().width;
    int h = getBounds().height;
    
    if (tl == null) tl = getImage("/com/andyfrank/mame/ui/images/corner_tl.png");
    if (tr == null) tr = getImage("/com/andyfrank/mame/ui/images/corner_tr.png");
    if (br == null) br = getImage("/com/andyfrank/mame/ui/images/corner_br.png");
    
    // Clear background
    //GradientPaint gradient = new GradientPaint(0, 0, bgStart, 0, h, bgStop);        
    //g.setPaint(gradient);
    //g.fillRect(0,0,w,h);
    
    g.drawImage(tl, -10, -10, observer);
    g.drawImage(tr, w+2, -10, observer);
    g.drawImage(br, w+2, h+2, observer);      
    
    if (snap != null)
    {
      int iw = snap.getWidth(observer);
      int ih = snap.getHeight(observer);
      
      if (iw > 350 || ih > 350)
      {
        double sx = (double)350 / (double)iw;
        double sy = (double)350 / (double)ih;
        double s = Math.min(sx, sy);
        
        int dw = (int)(iw * s);
        int dh = (int)(ih * s);
        int ix = (w - dw) / 2;
        int iy = (h - dh) / 2;
        
        g.drawImage(snap, 
          ix, iy, ix+dw, iy+dh, 
          0, 0, iw, ih,
          Color.black, observer);
      }
      else
      {
        int ix = (w-iw) / 2;
        int iy = (h-ih) / 2; //20;
        g.drawImage(snap, ix, iy, observer);
      }
    }
  }

////////////////////////////////////////////////////////////////
// SelectionListener
////////////////////////////////////////////////////////////////
  
  /**
   * Update details to new rom.
   */
  public void romSelected(int index, Rom rom)
  {
    StringBuffer buf = new StringBuffer();
    buf.append(Config.getMameHome()).append("/snap/").append(rom.getName()).append(".png"); 
    
    if (snap != null) snap.flush();
    snap = Toolkit.getDefaultToolkit().createImage(buf.toString());
    repaint();
  }  
  
////////////////////////////////////////////////////////////////
// ImageObserver
////////////////////////////////////////////////////////////////

  class Observer implements ImageObserver
  {
    public boolean imageUpdate(Image img, int info, int x, int y, int width, int height)
    {
      if ((info & ImageObserver.ALLBITS) != 0) 
      {
        repaint();
        return false;
      } 
      else return true;
    }
  }
  private Observer observer = new Observer();
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private Image snap = null;
  private Image tr;
  private Image tl;
  private Image br;
  
  private Color bgStart = new Color(0xeeeeee);
  private Color bgStop  = new Color(0xbbbbbb);
  //private Font font = new Font("Arial", Font.BOLD, 26); 
}

