/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**
 * IndexPane indicates the current index in the ROM list.
 */
public class IndexPane
  extends Widget
  implements SelectionListener
{
////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////

  public IndexPane(RomManager manager)
  {
    this.manager = manager;
  }
  
////////////////////////////////////////////////////////////////
// Layout
////////////////////////////////////////////////////////////////
  
  /**
   * Compute preferred size.
   */
  public void computePreferredSize()
  {    
    setPreferredSize(200,45);
  }

////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
 
  /**
   * Paint.
   */
  public void paint(Graphics2D g)
  {
    int w = getBounds().width;
    int h = getBounds().height;
    FontMetrics fm = g.getFontMetrics(font);
    
    String str = Config.getShowFavorites() ? "Favorites" : "All";
    str += "   " + index + " of " + manager.getSize(Config.getShowFavorites());
    int tx = w - fm.stringWidth(str) - 10;
    int ty = fm.getAscent() + ((h - fm.getHeight()) / 2);
    
    g.setColor(Color.white);    
    g.setFont(font);
    g.drawString(str, tx, ty);
  }

////////////////////////////////////////////////////////////////
// SelectionListener
////////////////////////////////////////////////////////////////
  
  public void romSelected(int index, Rom rom)
  {
    if (!Config.getShowFavorites()) 
    {
      this.index = index+1;
    }
    else
    {
      for(int i=0,j=0; i<manager.getSize(false); i++)
      {
        Rom r = manager.getRom(i);
        if (r.isFavorite())
        {
          if (i == index) 
          {
            this.index = j+1;
            break;
          }
          j++;
        }
      }
    }
    repaint();
  }

////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private int index = 1;
  private RomManager manager;  
  private Font font = new Font("Arial", Font.BOLD, 18);
}

