/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */ 
package com.andyfrank.mame.ui;
 
import java.awt.*;
import java.awt.event.*;

/**
 * Edits rom info.
 */
public class InfoEditor
{
////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////

  public InfoEditor(Component owner, RomManager manager, Rom rom)
  {    
    while (!(owner instanceof Frame))
      owner = owner.getParent();
    
    this.manager = manager;
    this.rom = rom;
    
    dlg = new Dialog((Frame)owner, "Edit"); 

    Button ok = new Button("Save");
    ok.setFont(labelFont);
    ok.addActionListener(new OkListener());
    
    Button cancel = new Button("Cancel");
    cancel.setFont(labelFont);
    cancel.addActionListener(new CancelListener());
    
    Label la = new Label("Rom name:", Label.LEFT);
    Label lb = new Label("Default name:", Label.LEFT);
    Label lc = new Label("Display name:", Label.LEFT);
    Label ld = new Label("Params:", Label.LEFT);
    la.setFont(labelFont);    
    lb.setFont(labelFont);    
    lc.setFont(labelFont);    
    ld.setFont(labelFont);
    
    ta = new TextField(rom.getName(), 50);
    ta.setEditable(false);
    ta.setFont(fieldFont);
    
    tb = new TextField(rom.getDefaultName(), 50);
    tb.setEditable(false);
    tb.setFont(fieldFont);
    
    tc = new TextField("", 50);
    if (rom.displayName != null) tc.setText(rom.displayName);
    tc.setFont(fieldFont);
    
    td = new TextField("", 50);
    if (rom.params != null) td.setText(rom.params);
    td.setFont(fieldFont);
    
    cc = new Checkbox("Favorite");
    cc.setState(rom.isFavorite());
    cc.setFont(labelFont);
        
    Panel q = new Panel();
    q.setLayout(new GridBagLayout());
    
    GridBagConstraints c = new GridBagConstraints();
    c.fill  = GridBagConstraints.HORIZONTAL;
   
    c.gridx = 0;
    c.gridy = 0;    
    q.add(la, c);
    
    c.gridx = 1;
    c.gridy = 0;    
    q.add(ta, c);
    
    c.gridx = 0;
    c.gridy = 1;
    q.add(lb, c);
    
    c.gridx = 1;
    c.gridy = 1;
    q.add(tb, c);
    
    c.gridx = 0;
    c.gridy = 2;
    q.add(lc, c);
    
    c.gridx = 1;
    c.gridy = 2;
    q.add(tc, c);

    c.gridx = 0;
    c.gridy = 3;
    q.add(ld, c);

    c.gridx = 1;
    c.gridy = 3;
    q.add(td, c);
    
    c.gridx = 0;
    c.gridy = 4;
    q.add(cc, c);
        
    Panel bottom = new Panel();
    bottom.setLayout(new FlowLayout(FlowLayout.CENTER));
    bottom.add(ok);
    bottom.add(cancel);
    
    Panel p = new Panel();
    p.setLayout(new BorderLayout());
    p.add(q, BorderLayout.CENTER);
    p.add(bottom, BorderLayout.SOUTH);
    p.addKeyListener(new KeyboardListener());
    
    Dimension size = Toolkit.getDefaultToolkit().getScreenSize();    
    int w = 800;
    int h = 270;
    int x = (size.width - w) / 2;
    int y = (size.height - h) / 2;
        
    dlg.setModal(true);
    dlg.setLayout(new BorderLayout());
    dlg.add(p, BorderLayout.CENTER);
    dlg.setBounds(x,y,w,h);    
  }                        
  
  public void doit()
  {
    dlg.show(); 
  }
  
  /**
   * Save changes back to rom.
   */
  public void saveChanges()
  {
    String s = tc.getText();
    String p = td.getText();
    rom.setDisplayName(s.length() == 0 ? null : s);
    rom.setParams(p.length() == 0 ? null : p);
    rom.setFavorite(cc.getState());
    manager.save();
  }  
  
////////////////////////////////////////////////////////////////
// Listeners
////////////////////////////////////////////////////////////////

  class OkListener implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      saveChanges();
      dlg.dispose();
    }
  }
  
  class CancelListener implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      dlg.dispose();
    }
  }

////////////////////////////////////////////////////////////////
// KeyListener
////////////////////////////////////////////////////////////////

  class KeyboardListener implements KeyListener
  {
    public void  keyPressed(KeyEvent event) 
    {
      switch (event.getKeyCode())
      {
        case KeyEvent.VK_ENTER:
          saveChanges();
          dlg.dispose();
          break;
        
        case KeyEvent.VK_ESCAPE: 
          dlg.dispose();
          break;                
      }
    }
    
    public void  keyReleased(KeyEvent event) {}
    public void  keyTyped(KeyEvent event) {} 
  } 
        
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private static Font labelFont = new Font("Verdana", Font.BOLD, 20);
  private static Font fieldFont = new Font("Verdana", Font.PLAIN, 20);
  
  private RomManager manager;
  private Rom rom;
  
  private Dialog dlg;
  private TextField ta,tb,tc,td;
  private Checkbox cc;
}

