/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * ListPane is the main interface widget.  It lists the available
 * games in a scrollable list.  The joystick up/down is used to scroll
 * the list.  Player 1 Button 1 (CTRL) starts the selected game.
 */
public class ListPane
  extends Widget
{
////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////
  
  /**
   * Create new ListPane using the specified RomManager.
   */  
  public ListPane(RomManager m)
  {    
    this.manager = m;    
  }

////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
 
  /**
   * Paint our list.
   */
  public void paint(Graphics2D g)
  {
    int w = getBounds().width;
    int h = getBounds().height;    
    FontMetrics fm = g.getFontMetrics(font);
    int dw = w-15;
    
    // Adjust row height
    // TODO    
    
    // Init  images
    if (mameImg == null) mameImg = getImage("/com/andyfrank/mame/ui/images/mame.png");
    if (tr == null) tr = getImage("/com/andyfrank/mame/ui/images/corner_tr.png");
    if (tl == null) tl = getImage("/com/andyfrank/mame/ui/images/corner_tl.png");
    if (br == null) br = getImage("/com/andyfrank/mame/ui/images/corner_br.png");
    if (bl == null) bl = getImage("/com/andyfrank/mame/ui/images/corner_bl.png");
                                                
    // Clear background
    //GradientPaint gradient = new GradientPaint(0, 0, bgStart, 0, h, bgStop);        
    //g.setPaint(gradient);
    //g.fillRect(0,0,w,h);
    
    g.drawImage(tl, -10, -10, observer);
    g.drawImage(bl, -10, h+2, observer);
    g.drawImage(br, w+2, h+2, observer);    
        
    // Scroll if nesseccary
    while (((selectedRow-offset) * rowHeight) < 0) offset--;
    while (((selectedRow-offset+1) * rowHeight) > h) offset++;
    
    // Paint scrollbar
    Rom[] roms = manager.getRomList(Config.getShowFavorites());
    double size = roms.length;
    int sh = (int)((h / rowHeight) / size * (double)h); 
    int sy = (int)((double)selectedRow / size * (double)(h-sh));
    if (sh < h)
    {
      g.setColor(scrollColor);
      g.fillRect(w-10, sy, 10, sh);
    }
        
    // Paint rows
    g.setFont(font);
    int y  = 0;
    int ty = ((rowHeight - fm.getHeight()) / 2) + fm.getAscent();
    int iy = (rowHeight - 32) / 2;
        
    for(int i=offset; i<size; i++)
    {
      if (i == selectedRow)
      {
        GradientPaint gradient = 
          new GradientPaint(0, y, gradientStart, 0, y+rowHeight, gradientStop);        
        g.setPaint(gradient);
        g.fillRect(0, y, dw, rowHeight);        
      }      
      
      if (!Config.getShowFavorites() && roms[i].isFavorite())
        g.setColor(favorite);
      else
        g.setColor(Color.white);      
      
      Shape old = g.getClip();
      g.clipRect(0,0,dw,y+rowHeight);
      g.drawImage(mameImg, 8, y+iy, observer);
      g.drawString(roms[i].getDisplayName(), 115, y+ty);
      g.setClip(old);
      y += rowHeight; 
      if (y + rowHeight > h) break;
    }
  } 
  
  /**
   * Get the selected row.
   */
  public int getSelectedRow()
  {
    if (!Config.getShowFavorites()) return selectedRow;
    
    int row = selectedRow;
    for(int i=0,j=0; i<manager.getSize(false); i++)
    {
      Rom r = manager.getRom(i);
      if (r.isFavorite())
      {
        if (j == selectedRow) return i;
        j++;
      }
    }
    
    return -1;
  }
  
////////////////////////////////////////////////////////////////
// Listeners
////////////////////////////////////////////////////////////////
  
  /**
   * Add the specifed SelectionListener.
   */
  public void addListener(SelectionListener l)
  {
    listeners.add(l);
  }
  
  /**
   * Remove the specified SelectionListener.
   */
  public void removeListener(SelectionListener l)
  {
    listeners.remove(l);
  }
  
  /**
   * Notify all listeners of new selection.
   */
  public void notifyListeners()
  {
    int row = getSelectedRow();
    Rom rom = manager.getRom(row);
    
    for(int i=0; i<listeners.size(); i++)
    {
      SelectionListener l = (SelectionListener)listeners.get(i);
      l.romSelected(row, rom);
    }
  }
  
////////////////////////////////////////////////////////////////
// ImageObserver
////////////////////////////////////////////////////////////////

  class Observer implements ImageObserver
  {
    public boolean imageUpdate(Image img, int info, int x, int y, int width, int height)
    {
      if ((info & ImageObserver.ALLBITS) != 0) 
      {
        // TODO: HACK
        // Call repaint on shell because corner 
        // images are outside bounds.
        getShell().repaint();
        return false;
      } 
      else return true;
    }
  }
  private Observer observer = new Observer();
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private RomManager manager;
  private ArrayList listeners = new ArrayList();
  
  private Font font = new Font("Arial", Font.BOLD, 26);
  private Color scrollColor   = new Color(0x000080);
  private Color gradientStart = new Color(0x0000ff);
  private Color gradientStop  = new Color(0x000080);
  private Color bgStart = Color.white;
  private Color bgStop  = new Color(0xbbbbbb);
  private Color favorite  = new Color(0xffff00);
  
  private Image mameImg;
  private Image tr;
  private Image tl;
  private Image br;
  private Image bl;
  
  private int rowHeight = 70;
  int offset = 0;  
  int selectedRow = 0;
}

