/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */ 
package com.andyfrank.mame.ui;

/**
 * Rom describes a ROM and parameters to pass to its emulator.
 */
public class Rom
{
////////////////////////////////////////////////////////////////
// Constructors 
////////////////////////////////////////////////////////////////
  
  /**
   * Create a new Rom with the specified internal name.
   */
  public Rom(String name)
  {    
    this.name = name;
    this.defaultName = name;
  }  
  
  /**
   * Create q new Rom with the specified internal and
   * display name.
   */
  public Rom(String name, String displayName)
  {
    this.name = name;
    this.displayName = displayName;
    this.defaultName = name;
  }

////////////////////////////////////////////////////////////////
// Access
////////////////////////////////////////////////////////////////

  /** Get ROM name. */
  public String getName() { return name; }
  
  /** Get display name. */
  public String getDisplayName() 
  { 
    if (displayName == null) 
      return defaultName; 
    return displayName;
  }
  public void setDisplayName(String dn) { this.displayName = dn; }
  
  /** Get default name. */
  public String getDefaultName() { return defaultName; }
  public void setDefaultName(String dn) { this.defaultName = dn; } 
  
  /** Get favorite flag. */
  public boolean isFavorite() { return favorite; }
  public void setFavorite(boolean b) { this.favorite = b; }
  
  /** Get params. */
  public String getParams() { return params; }
  public void setParams(String p) { this.params = p; }  
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////

  private String name;
  private String defaultName;
  String displayName = null;
  String params = null;
  private boolean favorite = false;
}

