/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**
 * Shell.
 */
public class Shell
  extends java.awt.Panel
{
////////////////////////////////////////////////////////////////
// Constructor                  
////////////////////////////////////////////////////////////////
 
  /**
   * Add this widget to the shell.
   */
  public Shell()
  {
    manager = new RomManager();
    File file = new File(Config.getMameHome() + "/roms");
    String[] files = file.list(new Filter());    
    for(int i=0; i<files.length; i++)
    {
      if (files[i].endsWith(".zip")) 
        files[i] = files[i].substring(0, files[i].length()-".zip".length());
      manager.addRom(new Rom(files[i]));
    }
    manager.loadMetaData("rominfo.xml");
    
    background = new Background(new Color(0x000080));
    background.setShell(this);
    
    logo = new Logo();
    logo.setShell(this);
    
    listBg = new Background(new Color(0x000040));
    listBg.setShell(this);

    snapBg = new Background(new Color(0x000040));
    snapBg.setShell(this);    

    details = new DetailsPane();
    details.setShell(this);
    
    index = new IndexPane(manager);
    index.setShell(this);
        
    list = new ListPane(manager);
    list.setShell(this);
    //list.addListener(index);
    list.addListener(details);
    list.addListener(index);
    
    kids = new Widget[] { background, logo, listBg, snapBg, list, details, index };
    addKeyListener(new KeyboardListener());
    
    // Start ScrollManager
    //new ScrollManager().start();
  }  

  static class Filter implements FilenameFilter
  {
    public boolean accept(File dir, String name)
    {
      return !name.equals("dir.txt");
    }
  }
   
////////////////////////////////////////////////////////////////
// Layout
////////////////////////////////////////////////////////////////
  
  /**
   * Layout child widgets.
   */
  public void layout()
  {
    int gap = 30;
    int w = getWidth();
    int h = getHeight();    
    
    background.setBounds(0,0,w,h);
    logo.setBounds(gap,gap-15,300,80);    
    snapBg.setBounds(w-390-gap,gap,390,390);
    listBg.setBounds(gap,80+gap,snapBg.getBounds().x-gap,h-80-(gap*3));
    
    Rectangle r = listBg.getBounds();
    list.setBounds(r.x+10, r.y+10, r.width-20, r.height-20);
    
    index.computePreferredSize();
    int iw = index.getPreferredWidth();
    int ih = index.getPreferredHeight();
    index.setBounds(r.x+r.width-iw-10, r.y-ih-5, iw, ih);

    r = snapBg.getBounds();
    details.setBounds(r.x+10, r.y+10, r.width-20, r.height-20);    
  }  

////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
  
  /**
   * Override update() to call paint() to reduce flicker.
   */
  public final void update(Graphics g) 
  { 
    paint(g);
  }  
  
  /**
   * Double-buffer graphics to reduce flicker.
   */
  public final void paint(Graphics graphics)
  {
    int w = getSize().width;
    int h = getSize().height;    
        
    // Create new buffer if neccesary
    if (buffer == null || buffer.getWidth(null) != w || buffer.getHeight(null) != h)
    {
      buffer = createImage(w, h);
      layout();
    }
      
    Graphics2D g = (Graphics2D)buffer.getGraphics();    
    g.setRenderingHint(
      RenderingHints.KEY_TEXT_ANTIALIASING, 
      RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    
    for(int i=0; i<kids.length; i++)
    {
      Rectangle bounds = kids[i].getBounds();
      g.translate(bounds.x, bounds.y);
      kids[i].paint(g);
      g.translate(-bounds.x, -bounds.y);  
    }    
    
    graphics.drawImage(buffer, 0, 0, null);
    g.dispose();
  }

////////////////////////////////////////////////////////////////
// KeyListener
////////////////////////////////////////////////////////////////

  class KeyboardListener implements KeyListener
  {
    public void  keyPressed(KeyEvent event) 
    {
      switch (event.getKeyCode())
      {
        case KEY_UP:
          if (list.selectedRow > 0) 
          {
            list.selectedRow -= scrollSpeed;
            if (list.selectedRow < 0) list.selectedRow = 0;
            list.notifyListeners();
            list.repaint();
          }
          isScrolling = true;
          break;
        
        case KEY_DOWN: 
          int size = manager.getSize(Config.getShowFavorites());
          if (list.selectedRow < size-1)
          {
            list.selectedRow += scrollSpeed;
            if (list.selectedRow >= size-1) list.selectedRow = size-1;
            list.notifyListeners();
            list.repaint();
          }
          isScrolling = true;
          break;
        
        case KEY_FAST_SCROLL:
          scrollSpeed = 25;
          break;
        
        case KEY_SELECT:
          manager.run(list.getSelectedRow());
          list.repaint();
          break;
          
        case KEY_TOGGLE_FAVS:
          boolean show = Config.getShowFavorites();
          if (show)
          {
            favPos = list.selectedRow;
            favOff = list.offset;
            list.selectedRow = allPos;
            list.offset = allOff;
          }
          else
          {
            allPos = list.selectedRow;
            allOff = list.offset;
            list.selectedRow = favPos;
            list.offset = favOff;
          }
          Config.setShowFavorites(!show);
          list.notifyListeners();
          repaint();
          break;
        
        case KEY_EDIT_ROM:
          new InfoEditor(Shell.this, manager, manager.getRom(list.getSelectedRow())).doit();
          list.repaint();
          break;          
      }
    }
    
    public void  keyReleased(KeyEvent event) 
    {
      if (event.getKeyCode() == KEY_FAST_SCROLL)
        scrollSpeed = 1;
    }
    
    public void  keyTyped(KeyEvent event) {} 
  } 
  
//  class ScrollManager extends Thread
//  {
//    public void run()
//    {
//      while (true)
//      {
//        try { Thread.sleep(rate); }
//        catch (Exception e) {}
//        
//        if (isScrolling)
//        {
//          time += rate;
//          switch (time)
//          {
//            case 1000: scrollSpeed = 2;  break;
//            case 2000: scrollSpeed = 4;  break;
//            case 3000: scrollSpeed = 50; break;            
//          }
//        }
//        else time = 0;
//      }
//    }
//    int rate = 100;
//    int time = 0;
//  }
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////

  private static final int KEY_UP          = KeyEvent.VK_UP;
  private static final int KEY_DOWN        = KeyEvent.VK_DOWN;
  private static final int KEY_FAST_SCROLL = KeyEvent.VK_CONTROL;
  private static final int KEY_SELECT      = KeyEvent.VK_1;
  private static final int KEY_TOGGLE_FAVS = KeyEvent.VK_F2;
  private static final int KEY_EDIT_ROM    = KeyEvent.VK_F7;
  
  private Image buffer;
  private Widget[] kids;
  
  private int scrollSpeed = 1;
  private boolean isScrolling = false; 
  private int allPos = 0;
  private int allOff = 0;
  private int favPos = 0;  
  private int favOff = 0;
    
  private RomManager manager;
  private Background background;
  private Logo logo;
  private Background listBg;
  private Background snapBg;
  private ListPane list;
  private DetailsPane details;
  private IndexPane index;
}



