/*
 * Copyright (c) 2004, Andy Frank
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *   
 *  - Neither the name of Andy Frank nor the names of its 
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package com.andyfrank.mame.ui;

import java.awt.*;
import java.awt.image.*;
import java.net.*;

/**
 * Widget.
 */
public class Widget
{
////////////////////////////////////////////////////////////////
// Layout
////////////////////////////////////////////////////////////////
  
  /**
   * Compute this widget's preferred size.
   */
  public void computePreferredSize()
  {    
    setPreferredSize(10,10);
  }
  
  /**
   * Get this widget's preferred width.
   */
  public int getPreferredWidth()
  {
    return prefsize.width;
  }
  
  /**
   * Get this widget's preferred height.
   */
  public int getPreferredHeight()
  {
    return prefsize.height;
  }
  
  /**
   * Get the bounds for this widget.
   */
  public Rectangle getBounds()
  {
    return bounds;
  }
  
  /**
   * Set the bounds for this widget.
   */
  public void setBounds(int x, int y, int w, int h)
  {
    bounds.x = x;
    bounds.y = y;
    bounds.width  = w;
    bounds.height = h;
  }  
  
  /**
   * Set the preferred size for this widget.
   */
  public void setPreferredSize(int w, int h)
  {
    prefsize.width  = w;
    prefsize.height = h;
  }  
  
  /**
   * Get the shell.
   */
  public Shell getShell()
  {
    return shell;
  }  
  
  /**
   * Set the shell.
   */
  void setShell(Shell shell)
  {
    this.shell = shell;
  }
  
////////////////////////////////////////////////////////////////
// Paint
////////////////////////////////////////////////////////////////
  
  /**
   * Repaint widget.
   */
  public final void repaint()
  {
    shell.repaint(bounds.x, bounds.y, bounds.width, bounds.height);
  }
  
  
  /**
   * Paint.
   */
  public void paint(Graphics2D g)
  {
  }

////////////////////////////////////////////////////////////////
// Image Loading
////////////////////////////////////////////////////////////////
  
  /**
   * Load image.
   */
  public Image getImage(String path)
  {
    //Image img = Toolkit.getDefaultToolkit().createImage(filename);
    //img.getWidth(observer);
    //return img;
    
    URL url = getClass().getResource(path);
    Image img = Toolkit.getDefaultToolkit().createImage(url);
    img.getWidth(observer);
    return img;    
  }  
  
  private class Observer implements ImageObserver
  {
    public boolean imageUpdate(Image img, int info, int x, int y, int width, int height)
    {
      if ((info & ImageObserver.ALLBITS) != 0) 
      {
        repaint();
        return false;
      } 
      else return true;
    }
  }
  private Observer observer = new Observer();
  
////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  
  private Shell shell;
  private Rectangle bounds = new Rectangle();
  private Dimension prefsize = new Dimension();
}



